# -*- coding: utf-8 -*-

# MIT License
#
# Copyright (c) 2021 Mehmet Umut Mutlu, Nazlıcan Yetimaslan, İlker Atagun
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import dizge.tools as tp


def standardize(data, preference=1):
    """

    :param data:
    :param preference:
    :return:
    """
    if isinstance(data, str):
        splittedText = data.split()
        normalizedText = [tp.normalize(word) for word in splittedText]
    else:
        normalizedText = [tp.normalize(word) for word in data]

    normalizedWords = [word[0] for word in normalizedText]
    loanSignals = [word[1] for word in normalizedText]
    originalWords = [word[2] for word in normalizedText]

    if preference == 1:
        return normalizedWords
    elif preference == 2:
        return loanSignals
    elif preference == 3:
        return originalWords
    else:
        return "ERROR: Your preference isn't an option!"


def analyze(data, mode):
    """
    araçlardan bir veya birkaçını bir veri seti üzerinde tek seferde analiz etmek ve bütüncül bir çıktı almak için
    `analyzer` modülü içindeki `analyze()` fonksiyonunu kullanabilirsiniz. Bu fonksiyon ilki verinin kendisi,
    ikincisi de istediğiniz araçların adlarından oluşan bir liste olmak üzere iki zorunlu parametreden oluşmaktadır.

    >>> from dizge.analyzer import analyze
    >>> data = pd.read_csv("../data/testset_v4.0.csv", sep=";", index_col="ID")
    >>> words = standardize(data["word"])

    >>> result = analyze(words, ["g2p", "harmony"])

    >>> result.to_csv("testset_v4.0.csv", sep=";", encoding="utf-8-sig")

    :param data: Analiz edilmek istenen csv dosyası
    :param mode: kullanmak istenilen aracın adı
    :return: csv dosyası
    """
    dict = {
            'word': data
            }
    if "g2p" in mode:
        result = [tp.g2p(word) for word in data]
        dict["g2p"] = result
    if "syllable_o" in mode:
        result = [tp.syllable_o(word) for word in data]
        dict["syllable_o"] = result
    if "syllable_p" in mode:
        result = [tp.syllable_p(word) for word in data]
        dict["syllable_p"] = result
    if "countSylable" in mode:
        result = [tp.countSyllable(word) for word in data]
        dict["countSylable"] = result
    if "harmony" in mode:
        result = [tp.vowelHarmony(word) for word in data]
        dict["harmony"] = result
    return dict



# -*- coding: utf-8 -*-

# MIT License
#
# Copyright (c) 2021 Mehmet Umut Mutlu, Nazlıcan Yetimaslan, İlker Atagun
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import dizge.competence.phonology as p
import dizge.competence.lexicon as lex

# NORMALIZATION
normalizationChars = {
    "!": "",
    '"': "",
    "#": "",
    "$": "",
    "%": "",
    "&": "",
    "\\": "",
    "'": "",
    "(": "",
    ")": "",
    "*": "",
    "+": "",
    ",": "",
    "-": "",
    ".": "",
    "/": "",
    ":": "",
    ";": "",
    "<": "",
    "=": "",
    ">": "",
    "?": "",
    "@": "",
    "[": "",
    "\\": "",
    "]": "",
    "^": "",
    "_": "",
    "`": "",
    "{": "",
    "|": "",
    "}": "",
    "~": ""
}

loanSignals = {
    "â": "a",
    "ê": "e",
    "î": "i",
    "ô": "ö",
    "û": "ü"
}


#

def softG(word):
    """

    :param word:
    :return:
    """
    softGCount = word.count("ğ")
    i = 0
    word2 = word

    while i < softGCount:
        indexSoftG = word2.index("ğ", 1)
        final = 1 if word2[-1] == "ğ" else 0

        previousP = indexSoftG - 1
        nextP = indexSoftG + 1

        if final == 1:
            word2 = word2.replace("ğ", "\u02D0", 1)
        elif "yığıl" in word2:
            word2 = word2.replace("yığıl", "yıoul", 1)
        elif p.isVowel(word2[previousP]) and p.isVowel(word2[nextP]):
            if (p.isUnrounded(word2[previousP]) and p.isFront(word2[previousP])) and (
                    p.isUnrounded(word2[nextP]) and p.isFront(word2[nextP])):
                word2 = word2.replace("ğ", "y", 1)
            elif word2[previousP] == word2[nextP]:
                word2 = word2.replace("ğ", "\u02D0", 1)
                word2 = word2[:nextP] + word2[nextP + 1:]
            elif word2[previousP] != word2[nextP]:
                word2 = word2.replace("ğ", "\u02D0", 1)
        elif p.isVowel(word2[previousP]) and p.isConsonant(word2[nextP]):
            if p.isUnrounded(word2[previousP]) and p.isFront(word2[previousP]):
                word2 = word2.replace("ğ", "y", 1)
            else:
                word2 = word2.replace("ğ", "\u02D0", 1)

        i += 1

    return word2


def normalize(word, mode="human"):
    """

    :param word:
    :param mode:
    :return:
    """
    loanAlert = 0
    word1 = word.strip().lower().replace(" ", "")
    word2 = ""
    for letter in word1:
        if letter in normalizationChars.keys():
            word2 += normalizationChars[letter]
        else:
            word2 += letter

    word3 = ""
    for letter in word2:
        if letter in loanSignals.keys():
            word3 += loanSignals[letter]
            loanAlert = 1
        else:
            word3 += letter

    output = word3
    if "ğ" in word3 and mode == "machine":
        output = softG(word3)

    return output, loanAlert, word


# SYLLABLE ANALYZER
def syllable_o(word, mode="human"):
    """

    :param word:
    :param mode:
    :return:
    """
    vowels = ["a", "\u0251", "e", "\u025B", "\u0069", "I", "\u0268", "o", "\u0254", "\u00F8", "\u0153", "u", "U", "Y",
              "y"]

    def word2pattern(syl):
        """

        :param syl:
        :return:
        """
        seq = ""
        for phoneme in syl:
            if phoneme in vowels:
                seq += "V"
            else:
                seq += "C"
        if "VV" in seq:
            seq = seq.replace("VV", "V")
        return seq

    seq = list(normalize(word, mode)[0])
    for i in range(len(seq) - 1, -1, -1):
        if seq[i] in [v.grapheme for v in p.vowels]:
            if (i - 1) <= 0:
                continue
            elif seq[i - 1] in [v.grapheme for v in p.vowels]:
                continue
            else:
                seq[i - 1] = "." + seq[i - 1]

    analyzedWord = "".join(seq)

    if "." in analyzedWord:
        analyzedWord = analyzedWord.split(".")
    else:
        analyzedWord = [analyzedWord]

    if mode == "machine":
        analyzedWord_seq = [word2pattern(syl) for syl in analyzedWord]

    if mode == "machine":
        return list(zip(analyzedWord, analyzedWord_seq))
    else:
        return analyzedWord


def syllable_p(word):
    """

    :param word:
    :return:
    """
    vowels = ["a", "\u0251", "e", "\u025B", "\u0069", "I", "\u0268", "o", "\u0254", "\u00F8", "\u0153", "u", "U", "Y",
              "y"]

    def word2pattern(syl):
        """

        :param syl:
        :return:
        """
        if "d\u0292" in syl:
            syl = syl.replace("d\u0292", "1")
        elif "t\u0283" in syl:
            syl = syl.replace("t\u0283", "2")
        elif "z\u0325" in syl:
            syl = syl.replace("z\u0325", "3")

        seq = ""
        for phoneme in syl:
            if phoneme in vowels:
                seq += "V"
            elif phoneme == "\u02D0" or phoneme == "\u02B0":
                continue
            else:
                seq += "C"
        if "VVV" in seq:
            seq = seq.replace("VVV", "V")
        elif "VV" in seq:
            seq = seq.replace("VV", "V")
        return seq

    if isinstance(g2p(word), str):
        output = g2p(word)
    else:
        output = g2p(word)[0]

    if "d\u0292" in output:
        output = output.replace("d\u0292", "1")
    elif "t\u0283" in output:
        output = output.replace("t\u0283", "2")
    elif "p\u02B0" in output:
        output = output.replace("p\u02B0", "3")
    elif "t\u02B0" in output:
        output = output.replace("t\u02B0", "4")
    elif "k\u02B0" in output:
        output = output.replace("k\u02B0", "5")
    elif "c\u02B0" in output:
        output = output.replace("c\u02B0", "6")

    seq = list(output)

    for i in range(len(seq) - 1, -1, -1):
        if seq[i] in vowels:
            if (i - 1) <= 0:
                continue
            elif seq[i - 1] in vowels or seq[i - 1] == "\u02D0" or seq[i - 1] == "\u02B0":
                continue
            else:
                seq[i - 1] = "." + seq[i - 1]

    analyzedWord = "".join(seq)

    if "1" in analyzedWord:
        analyzedWord = analyzedWord.replace("1", "d\u0292")
    elif "2" in analyzedWord:
        analyzedWord = analyzedWord.replace("2", "t\u0283")
    elif "3" in analyzedWord:
        analyzedWord = analyzedWord.replace("3", "p\u02B0")
    elif "4" in analyzedWord:
        analyzedWord = analyzedWord.replace("4", "t\u02B0")
    elif "5" in analyzedWord:
        analyzedWord = analyzedWord.replace("5", "k\u02B0")
    elif "6" in analyzedWord:
        analyzedWord = analyzedWord.replace("6", "c\u02B0")

    if "." in analyzedWord:
        analyzedWord = analyzedWord.split(".")
    else:
        analyzedWord = [analyzedWord]

    analyzedWord_seq = [word2pattern(syl) for syl in analyzedWord]

    return list(zip(analyzedWord, analyzedWord_seq))


def findMyIndex(syllableCounter, phonemeCounter, seq):
    """

    :param syllableCounter:
    :param phonemeCounter:
    :param seq:
    :return:
    """
    total = 0
    n = 0
    while n < (syllableCounter - 1):
        total += len(seq[n])
        n += 1
    total += phonemeCounter
    return (total - 1)


def g2p(word):
    word2 = word
    """

    :param word:
    :return:
    """
    if "zs" in word:
        word = word.replace("zs", "ss")
    syllables = syllable_o(softG(word), "machine")
    seq = [syllable[0] for syllable in syllables]
    syllableCounter = 0
    phonemeCounter = 0
    output = ""
    alternative = ""
    for syl in seq:
        syllableCounter += 1

        for phoneme in syl:
            phonemeCounter += 1

            if phoneme == "\u02D0":
                output += "\u02D0"

            elif phoneme == "\u02D0":
                output += "\u02D0"

            elif phoneme == "p":
                if p.isInitialWord(syllableCounter, phonemeCounter):
                    output += "p\u02B0"
                else:
                    output += "p"

            elif phoneme == "b":
                output += "b"

            elif phoneme == "t":
                if p.isInitialWord(syllableCounter, phonemeCounter):
                    output += "t\u02B0"
                else:
                    output += "t"

            elif phoneme == "d":
                output += "d"

            elif phoneme == "k":
                if p.getTongueSyllable(syl) == 1:
                    output += "c"
                else:
                    output += "k"
                if p.isInitialWord(syllableCounter, phonemeCounter):
                    output += "\u02B0"

            elif phoneme == "g":
                if p.getTongueSyllable(syl) == 1:
                    output += "\u025F"
                else:
                    output += "g"

            elif phoneme == "f":
                output += "f"

            elif phoneme == "v":
                if p.isInitialWord(syllableCounter, phonemeCounter) == False and p.isFinalWord(syllableCounter,
                                                                                               phonemeCounter,
                                                                                               seq) == False:
                    indexV = findMyIndex(syllableCounter, phonemeCounter, seq)
                    previousP = indexV - 1
                    nextP = indexV + 1
                    if p.isVowel(word[previousP]) and p.isVowel(word[nextP]):
                        output += "\u028B"
                    else:
                        output += "v"
                else:
                    output += "v"

            elif phoneme == "s":
                output += "s"

            elif phoneme == "z":
                if p.isFinalWord(syllableCounter, phonemeCounter, seq):
                    output += "z\u0325"
                else:
                    output += "z"

            elif phoneme == "ş":
                output += "\u0283"

            elif phoneme == "j":
                output += "\u0292"

            elif phoneme == "ç":
                output += "t\u0283"

            elif phoneme == "c":
                output += "d\u0292"

            elif phoneme == "h":
                if p.getTongueSyllable(syl) == 1:
                    output += "ç"
                else:
                    output += "x"

            elif phoneme == "y":
                output += "j"

            elif phoneme == "m":
                output += "m"

            elif phoneme == "n":
                if p.isFinalWord(syllableCounter, phonemeCounter, seq):
                    output += "n"
                else:
                    indexN = findMyIndex(syllableCounter, phonemeCounter, seq)
                    nextN = indexN + 1
                    if word[nextN] in ["v", "f", "k"] and word in ["cankurtaran", "enfes", "erkenvarmak", "envanter"]:
                        output += "\u0271"
                    elif word[nextN] in ["k", "g"] and word not in ["cankurtaran", "enfes", "erkenvarmak", "envanter"]:
                        output += "\u014B"
                    else:
                        output += "n"

            elif phoneme == "r":
                if p.isInitialWord(syllableCounter, phonemeCounter):
                    output += "r"
                elif p.isFinalWord(syllableCounter, phonemeCounter, seq):
                    output += "\u0263"
                else:
                    output += "\u027e"

            elif phoneme == "l":
                if p.getTongueSyllable(syl) == 1 or (
                        p.getTongueSyllable(syl) == 3 and p.isInitialWord(syllableCounter, phonemeCounter)):
                    output += "l"
                elif word == "gol":
                    output += "l"
                else:
                    output += "\u0142"

            elif phoneme == "a":
                output += "\u0251"

            elif phoneme == "e":
                if "kendi" in word or "belki" in word:
                    output += "\u025B"
                elif word.count(phoneme) > 1 and not ("kendi" in word or "belki" in word):
                    if output.count("e") > 0:
                        output += "\u025B"
                    else:
                        output += "e"
                else:
                    if syllableCounter > 1 or len(seq) == 1:
                        output += "\u025B"
                    else:
                        output += "e"

            elif phoneme == "o":
                indexO = findMyIndex(syllableCounter, phonemeCounter, seq)
                nextO = indexO + 1
                if p.isFinalWord(syllableCounter, phonemeCounter, seq) == False:
                    if word[nextO] == "ğ":
                        output += "o"
                    else:
                        output += "\u0254"
                else:
                    output += "\u0254"

            elif phoneme == "ö":
                if "öğe" in word or "öğü" in word or "öğr" in word or "öğl" in word or "öğm" in word:
                    output += "\u00F8"
                else:
                    output += "\u0153"

            elif phoneme == "i":
                indexİ = findMyIndex(syllableCounter, phonemeCounter, seq)
                nextİ = indexİ + 1
                if p.isFinalWord(syllableCounter, phonemeCounter, seq) == False:
                    if word[nextİ] == "ğ":
                        output += "\u0069"
                    else:
                        output += "I"
                else:
                    output += "I"

            elif phoneme == "ı":
                output += "\u0268"

            elif phoneme == "u":
                indexU = findMyIndex(syllableCounter, phonemeCounter, seq)
                nextU = indexU + 1
                if p.isFinalWord(syllableCounter, phonemeCounter, seq) == False:
                    if word[nextU] == "ğ":
                        output += "u"
                    else:
                        output += "U"
                else:
                    if softG(word)[-1] == "\u02D0":
                        output += "U"
                    else:
                        output += "u"

            elif phoneme == "ü":
                indexÜ = findMyIndex(syllableCounter, phonemeCounter, seq)
                nextÜ = indexÜ + 1
                if p.isFinalWord(syllableCounter, phonemeCounter, seq) == False:
                    if word[nextÜ] == "ğ":
                        output += "y"
                    else:
                        output += "Y"
                else:
                    output += "Y"

        phonemeCounter = 0
    syllableCounter = 0

    if "eğ" in word and "ejI" not in output:
        alternative = output
        alternative = alternative.replace("ej", "\u025B\u02D0I")
    elif "ejI" in output:
        alternative = output
        alternative = alternative.replace("ejI", "\u025B\u02D0I")
    elif "\u025Bj" in output:
        output = output.replace("\u025Bj", "\u025B\u02D0I")
    elif word.startswith("yığıl"):
        output = output.replace("j\u0268\u0254U\u0142", "j\u0268\u02D0o\u02D0U\u0142")
    else:
        if "\u0251j" in output and "mayo" not in word:
            output = output.replace("\u0251j", "\u0251\u02D0I")

        if "ijɛ" in output:
            output = output.replace("ijɛ", "I\u02D0\u025B")
        elif "ij" in output:
            output = output.replace("ij", "i\u02D0")
        elif "Ij" in output and "yy" not in word:
            output = output.replace("Ij", "i\u02D0")

        if "\u0254j" in output:
            output = output.replace("\u0254j", "o\u02D0I")

        if "\u0153j" in output:
            output = output.replace("\u0153j", "\u00F8\u02D0I")

        if "Uj" in output:
            output = output.replace("Uj", "U\u02D0I")

    if "\u0251x\u0251" in output:
        alternative = output
        alternative = alternative.replace("\u0251x\u0251", "\u0251\u02D0")
    elif "x\u0251n\u025B" in output:
        alternative = output
        alternative = alternative.replace("x\u0251n\u025B", "\u0251\u02D0n\u025B")
    elif "kahve" in word:
        alternative = output
        alternative = alternative.replace("\u0251xv", "\u0251\u02D0v")

    if output[-1] == "i":
        output = output[:-1] + "I"

    if output[-1] == "u":
        output = output[:-1] + "U"

    for loan in lex.loans.keys():
        if word.startswith(loan):
            output = [ph for ph in output]
            for change in lex.loans[loan]:
                output[change[1]] = change[0]
            output = "".join(output)

    if alternative != "":
        return output, alternative
    else:
        return output


def vowelHarmony(word):
    """

    :param word:
    :return:
    """
    def harmonyOne(word):
        """

        :param word:
        :return:
        """
        pattern = []
        for phoneme in word:
            if p.isBack(phoneme):
                pattern += "B"
            elif p.isFront(phoneme):
                pattern += "F"
            else:
                continue
        if "B" in pattern and "F" in pattern:
            return False
        else:
            return True

    def harmonyTwo(word):
        """

        :param word:
        :return:
        """
        result = True

        pattern = []
        for phoneme in word:
            if p.isVowel(phoneme):
                pattern += phoneme
            else:
                continue

        idxList = []
        for idx in range(len(pattern) - 1):
            idxList.append([idx, idx + 1])

        for i in idxList:
            if p.isUnrounded(pattern[i[0]]) and p.isUnrounded(pattern[i[1]]):
                result = True
            elif p.isRounded(pattern[i[0]]) and ((p.isRounded(pattern[i[1]]) and p.isClose(pattern[i[1]])) or (
                    p.isUnrounded(pattern[i[1]]) and p.isOpen(pattern[i[1]]))):
                result = True
            else:
                result = False
                break

        return result

    return (harmonyOne(word), harmonyTwo(word))


def countSyllable(word):
    """

    :param word:
    :return:
    """
    seq = syllable_p(word)
    patterns = {}
    for pattern in seq:
        if pattern[1] not in patterns.keys():
            patterns[pattern[1]] = 0
    for pattern in seq:
        patterns[pattern[1]] += 1
    return patterns




import pathlib
from setuptools import setup, find_packages

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text(encoding="utf-8")

# This call to setup() does all the work
setup(
    name="dizge",
    version="0.1.1",
    description="The OOP based grammar analyzer for Turkish",
    long_description=README,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url="https://github.com/mehmetumutmutlu/dizge",
    author="Mehmet Umut Mutlu, Nazlıcan Yetimaslan, İlker Atagün",
    author_email="mehmetumutmutlu@gmail.com",
    license="MIT",
    keywords=['python', 'nlp', 'linguistics', 'phonology', "turkish", "grammar"],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Intended Audience :: Developers",
        "Intended Audience :: Education",
        "Intended Audience :: Information Technology",
        "Intended Audience :: Science/Research",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ]
)


# -*- coding: utf-8 -*-

# MIT License
#
# Copyright (c) 2021 Mehmet Umut Mutlu, Nazlıcan Yetimaslan, İlker Atagun
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

loans = {
    "alaka": [["a\u02D0", 2]],
    "hakim": [["\u0251\u02D0", 1]],
    "memur": [["e\u02D0", 1]],
    "temin": [["e\u02D0", 2]],
    "tecil": [["e\u02D0", 2]],
    "tesir": [["e\u02D0", 2]],
    "itibaren": [["i\u02D0", 0]],
    "ilan": [["i\u02D0", 0], ["a", 2]],
    "lider": [["i\u02D0", 1]],
    "muaf": [["u\u02D0", 1]],
    "ödlek": [["t", 1]],
    "tahin": [["\u0251\u02D0", 2]],
    "nisan": [["i\u02D0", 1]],
    "niçin": [["i\u02D0", 1]],
    "rakım": [["\u0251\u02D0", 1]],
    "lüfer": [["y\u02D0", 1]],
    "kağıt": [["c", 0], ["a", 2]],
    "lamba": [["a", 1]],
    "kip": [["k", 0]]
    }

# loans2 = {
#     "alaka": "\u0251\u0142a\u02D0k\u0251",
#     "hakim": "x\u0251\u02D0cIm",
#     "memur": "me\u02D0mU\u0263",
#     "temin": "t\u02B0e\u02D0mIn",
#     "tecil": "t\u02B0e\u02D0d\u0292Il",
#     "tesir": "t\u02B0e\u02D0sI\u0263",
#     "itibaren": "i\u02D0tIb\u0251r\u025Bn",
#     "ilan": "i\u02D0\u0142an",
#     "lider": "li\u02D0d\u025B\u0263",
#     "muaf": "mu\u02D0\u0251f",
#     "ödlek": "\u0153tl\u025Bc",
#     "tahin": "t\u02B0\u0251\u02D0çIn",
#     "nisan": "ni\u02D0s\u0251n",
#     "niçin": "ni\u02D0t\u0283In",
#     "rakım": "r\u0251\u02D0k\u0268m",
#     "lüfer": "ly\u02D0f\u025B\u0263"
#     }


# -*- coding: utf-8 -*-

# MIT License
#
# Copyright (c) 2021 Mehmet Umut Mutlu, Nazlıcan Yetimaslan, İlker Atagun
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import dizge.tools.phonology as tp


# PHONOLOGICAL STRUCTURES
class Phoneme:
    """

    """
    def __init__(self, phonemeID, grapheme):
        """

        :param phonemeID:
        :param grapheme:
        """
        self.phonemeID = phonemeID  # her bir sesbirime ait benzersiz kimlik
        self.grapheme = grapheme  # yazıbirim


class Consonant(Phoneme):
    """

    """
    def __init__(self, phonemeID, grapheme, manner, place, voicing):
        """

        :param phonemeID:
        :param grapheme:
        :param manner:
        :param place:
        :param voicing:
        """
        Phoneme.__init__(self, phonemeID, grapheme)
        self.manner = manner  # çıkış biçimi
        self.place = place  # çıkış yeri
        self.voicing = voicing  # ötümlülük

    def getManner(self):
        """

        :return:
        """
        if self.manner == 1:
            return "patlamalı"  # plosive
        elif self.manner == 2:
            return "genizsil"  # nasal
        elif self.manner == 3:
            return "titrek"  # trill
        elif self.manner == 4:
            return "dokunmalı veya çarpmalı"  # tap or flap
        elif self.manner == 5:
            return "sürtünmeli"  # fricative
        elif self.manner == 6:
            return "yan sürtünmeli"  # lateral fricative
        elif self.manner == 7:
            return "sürtünmesiz"  # approximant
        elif self.manner == 8:
            return "yan sürtünmesiz"  # lateral approximant
        else:
            return "hata"  # error

    def getPlace(self):
        """

        :return:
        """
        if self.place == 1:
            return "çift-dudaksıl"  # bilabial
        elif self.place == 2:
            return "dişsil-dudaksıl"  # labiodental
        elif self.place == 3:
            return "dişsil"  # dental
        elif self.place == 4:
            return "diş-yuvasıl"  # alveolar
        elif self.place == 5:
            return "artdiş-yuvasıl"  # postalveolar
        elif self.place == 6:
            return "üstdamaksıl"  # retroflex
        elif self.place == 7:
            return "damaksıl"  # palatal
        elif self.place == 8:
            return "artdamaksıl"  # velar
        elif self.place == 9:
            return "küçük-dilsil"  # uvular
        elif self.place == 10:
            return "yutaksıl"  # pharyngeal
        elif self.place == 11:
            return "gırtlaksıl"  # glottal
        else:
            return "hata"  # error

    def getVoicing(self):
        """

        :return:
        """
        if self.voicing == 0:
            return "ötümsüz"  # voiceless
        elif self.voicing == 1:
            return "ötümlü"  # voiced
        else:
            return "hata"  # error


class Vowel(Phoneme):
    """

    """
    def __init__(self, phonemeID, grapheme, rounding, mandible, tongue):
        """

        :param phonemeID:
        :param grapheme:
        :param rounding:
        :param mandible:
        :param tongue:
        """
        Phoneme.__init__(self, phonemeID, grapheme)
        self.rounding = rounding
        self.mandible = mandible
        self.tongue = tongue

    def getRounding(self):
        """

        :return:
        """
        if self.rounding == 0:
            return "düz"  # unrounded
        elif self.rounding == 1:
            return "yuvarlak"  # rounded
        else:
            return "hata"  # error

    def getMandible(self):
        """

        :return:
        """
        if self.mandible == 1:
            return "kapalı"  # close
        elif self.mandible == 2:
            return "yarı-kapalı"  # close-mid
        elif self.mandible == 3:
            return "yarı-açık"  # open-mid
        elif self.mandible == 4:
            return "açık"  # open
        else:
            return "hata"  # error

    def getTongue(self):
        """

        :return:
        """
        if self.tongue == 1:
            return "öndil"  # front
        elif self.tongue == 2:
            return "ortadil"  # central
        elif self.tongue == 3:
            return "arkadil"  # back
        else:
            return "hata"  # error

    # def getHeight(self):
    #     if self.height == 0:
    #         return "alçak" # low
    #     elif self.height == 1:
    #         return "yüksek" # high
    #     else:
    #         return "hata" # error


# PHONOLOGICAL UNITS
# VOWELS
v001 = Vowel("v001", "a", 0, 4, 3)
v002 = Vowel("v002", "e", 0, 4, 1)
v003 = Vowel("v003", "o", 1, 4, 3)
v004 = Vowel("v004", "ö", 1, 4, 1)
v005 = Vowel("v005", "i", 0, 1, 1)
v006 = Vowel("v006", "ı", 0, 1, 2)
v007 = Vowel("v007", "u", 1, 1, 3)
v008 = Vowel("v008", "ü", 1, 1, 1)
vowels = [v001, v002, v003, v004, v005, v006, v007, v008]

# CONSONANTS
c001 = Consonant("c001", "p", 1, 1, 0)
c002 = Consonant("c002", "b", 1, 1, 1)
c003 = Consonant("c003", "t", 1, 3, 0)
c004 = Consonant("c004", "d", 1, 3, 1)
c005 = Consonant("c005", "k", 1, 8, 0)
c006 = Consonant("c006", "g", 1, 8, 1)
c007 = Consonant("c007", "f", 5, 2, 0)
c008 = Consonant("c008", "v", 5, 2, 1)
c009 = Consonant("c009", "s", 5, 3, 0)
c010 = Consonant("c010", "z", 5, 3, 1)
c011 = Consonant("c011", "ş", 5, 5, 0)
c012 = Consonant("c012", "j", 5, 5, 1)
c013 = Consonant("c013", "ç", 1, 5, 0)
c014 = Consonant("c014", "c", 1, 5, 1)
c015 = Consonant("c015", "h", 5, 11, 0)
c016 = Consonant("c016", "y", 7, 7, 1)
c017 = Consonant("c017", "m", 2, 1, 1)
c018 = Consonant("c018", "n", 2, 3, 1)
c019 = Consonant("c019", "r", 4, 4, 1)
c020 = Consonant("c020", "l", 8, 5, 1)
consonants = [c001, c002, c003, c004, c005, c006, c007, c008, c009, c010,
              c011, c012, c013, c014, c015, c016, c017, c018, c019, c020]


# FEATURE FUNCTIONS
def isVowel(phoneme):
    """

    :param phoneme:
    :return:
    """
    if phoneme in [i.grapheme for i in vowels]:
        return True
    else:
        return False


def isUnrounded(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.rounding for i in vowels if i.grapheme == phoneme] == [0]:
        return True
    else:
        return False


def isRounded(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.rounding for i in vowels if i.grapheme == phoneme] == [1]:
        return True
    else:
        return False


def isClose(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.mandible for i in vowels if i.grapheme == phoneme] == [1]:
        return True
    else:
        return False


def isCloseMid(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.mandible for i in vowels if i.grapheme == phoneme] == [2]:
        return True
    else:
        return False


def isOpenMid(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.mandible for i in vowels if i.grapheme == phoneme] == [3]:
        return True
    else:
        return False


def isOpen(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.mandible for i in vowels if i.grapheme == phoneme] == [4]:
        return True
    else:
        return False


def isFront(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.tongue for i in vowels if i.grapheme == phoneme] == [1]:
        return True
    else:
        return False


def isCentral(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.tongue for i in vowels if i.grapheme == phoneme] == [2]:
        return True
    else:
        return False


def isBack(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.tongue for i in vowels if i.grapheme == phoneme] == [3]:
        return True
    else:
        return False


def isConsonant(phoneme):
    """

    :param phoneme:
    :return:
    """
    if phoneme in [i.grapheme for i in consonants]:
        return True
    else:
        return False


def isPlosive(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [1]:
        return True
    else:
        return False


def isNasal(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [2]:
        return True
    else:
        return False


def isTrill(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [3]:
        return True
    else:
        return False


def isTaporFlap(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [4]:
        return True
    else:
        return False


def isFricative(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [5]:
        return True
    else:
        return False


def isLateralFricative(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [6]:
        return True
    else:
        return False


def isApproximant(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [7]:
        return True
    else:
        return False


def isLateralApproximant(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.manner for i in consonants if i.grapheme == phoneme] == [8]:
        return True
    else:
        return False


def isBilabial(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [1]:
        return True
    else:
        return False


def isLabiodental(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [2]:
        return True
    else:
        return False


def isDental(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [3]:
        return True
    else:
        return False


def isAlveolar(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [4]:
        return True
    else:
        return False


def isPostalveolar(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [5]:
        return True
    else:
        return False


def isRetroflex(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [6]:
        return True
    else:
        return False


def isPalatal(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [7]:
        return True
    else:
        return False


def isVelar(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [8]:
        return True
    else:
        return False


def isUvular(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [9]:
        return True
    else:
        return False


def isPharyngeal(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [10]:
        return True
    else:
        return False


def isGlottal(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.place for i in consonants if i.grapheme == phoneme] == [11]:
        return True
    else:
        return False


def isVoiced(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.voicing for i in consonants if i.grapheme == phoneme] == [1]:
        return True
    else:
        return False


def isVoiceless(phoneme):
    """

    :param phoneme:
    :return:
    """
    if [i.voicing for i in consonants if i.grapheme == phoneme] == [0]:
        return True
    else:
        return False


def isInitialWord(syllableCounter, phonemeCounter):
    """

    :param syllableCounter:
    :param phonemeCounter:
    :return:
    """
    if syllableCounter == 1 and phonemeCounter == 1:
        return True
    else:
        return False


def isFinalWord(syllableCounter, phonemeCounter, seq):
    """

    :param syllableCounter:
    :param phonemeCounter:
    :param seq:
    :return:
    """
    length = 0
    for syl in seq:
        length += len(syl)
    currentIndex = tp.findMyIndex(syllableCounter, phonemeCounter, seq)
    if length - 1 == currentIndex:
        return True
    else:
        return False


def getTongueSyllable(syllable):
    """

    :param syllable:
    :return:
    """
    for phoneme in syllable:
        if isVowel(phoneme):
            feature = [i.tongue for i in vowels if i.grapheme == phoneme][0]
    return feature
